use strict;
use warnings;
use Test::More tests => 6;

use_ok 'AnyEvent::Ident';
use_ok 'AnyEvent::Ident::Client';
use_ok 'AnyEvent::Ident::Request';
use_ok 'AnyEvent::Ident::Response';
use_ok 'AnyEvent::Ident::Server';
use_ok 'AnyEvent::Ident::Transaction';
